import React, { useState, useEffect } from 'react'
import { Table, Space, message } from 'antd'
import moment from 'moment'
import { useNavigate, useLocation } from 'react-router-dom'
import { getLogList } from '../services/index'

const LogTable = () => {
  // 初始化日志列表
  const [logList, setLogList] = useState([])
  // 表格Loading状态
  const [dataLoading, setDataLoading] = useState(false)

  const navigate = useNavigate()

  const { state } = useLocation()

  const changeTask = () => {
    message.success('终止成功')
  }
  const checkLog = id => {
    navigate('/task-log/log-watcher', { state: { id } })
  }
  const columns = [
    {
      title: '任务名称',
      dataIndex: 'jobName',
      key: 'jobName',
    },
    {
      title: '任务类型',
      dataIndex: 'taskType',
      key: 'taskType',
    },
    {
      title: '任务标签',
      dataIndex: 'taskTag',
      key: 'taskTag',
    },
    {
      title: '调度时间',
      dataIndex: 'triggerTime',
      key: 'triggerTime',
    },
    {
      title: '调度结果',
      dataIndex: 'triggerResult',
      key: 'triggerResult',
      render: code => (
        <span
          style={{
            color: code === 0 ? '#FF4D4F' : code === 1 ? '#52C41A' : '#4A4A4A',
          }}>
          {code === 0 ? '失败' : code === 1 ? '成功' : '无'}
        </span>
      ),
    },
    {
      title: '执行时间',
      dataIndex: 'handleTime',
      key: 'handleTime',
    },
    {
      title: '执行结果',
      dataIndex: 'handleResult',
      key: 'handleResult',
      render: code => (
        <span
          style={{
            color: code === 0 ? '#FF4D4F' : code === 1 ? '#52C41A' : '#4A4A4A',
          }}>
          {code === 0 ? '失败' : code === 1 ? '成功' : '无'}
        </span>
      ),
    },
    {
      title: '操作',
      key: 'operation',
      render: (_, record) => (
        <Space size="middle">
          <span
            onClick={() => {
              checkLog(record.key)
            }}
            style={{ color: '#1881DA', cursor: 'pointer' }}>
            查看
          </span>
          <span
            onClick={() => {
              changeTask(record.key)
            }}
            style={{ color: '#1881DA', cursor: 'pointer' }}>
            终止任务
          </span>
        </Space>
      ),
    },
  ]

  const fetchJoblog = async () => {
    setDataLoading(true)
    const { data } = await getLogList(state?.id)
    if (data.code === 200) {
      const list = data.data.map(item => {
        return {
          key: item.id,
          jobName: item.job_name,
          triggerTime: moment(item.trigger_time * 1000).format(
            'YYYY.MM.DD HH:MM'
          ),
          triggerResult: item.trigger_result,
          handleTime: moment(item.executor_time * 1000).format(
            'YYYY.MM.DD HH:MM'
          ),
          handleResult: item.executor_result,
          taskType: item.job_type,
          taskTag: item.job_tag,
        }
      })
      setLogList(list)
    }
    setDataLoading(false)
  }

  useEffect(() => {
    fetchJoblog()
  }, [])

  return (
    <Table
      columns={columns}
      dataSource={logList}
      bordered
      loading={dataLoading}
    />
  )
}

export default LogTable
