import React, { useState, useEffect } from 'react'
import { useNavigate } from 'react-router'
import {
  Table,
  Switch,
  Space,
  Button,
  message,
  Popconfirm,
  Dropdown,
  Menu,
  Popover,
} from 'antd'
import { EllipsisOutlined } from '@ant-design/icons'
import { Link } from 'react-router-dom'
import styled from 'styled-components'
import errImgUrl from '../style/img/err.png'
import successImgUrl from '../style/img/success.png'
import {
  getJmJobList,
  deleteJmJob,
  executeJmJob,
  updateJmJobStatus,
  getCronNext,
} from '../services'
import copy from 'copy-to-clipboard'

const TaskMgmtWrapper = styled.div`
  padding: 20px;
  .icon_wrapper {
    margin-right: 15px;
  }
  .link_btn {
    display: flex;
    justify-content: end;
    margin-bottom: 20px;
  }
`

const TaskMgmtView = () => {
  // 表格Loading状态
  const [dataLoading, setDataLoading] = useState(false)
  // 初始化列表
  const [jobList, setJogList] = useState([])

  const [recordKey, setRecordKey] = useState(null)

  const [dayData, setDayData] = useState([])

  const navigate = useNavigate()

  // 切换状态
  const changeTaskState = (checked, id) => {
    updateJmJobStatus({ id, status: checked ? 1 : 0 })
  }
  //执行
  const runTimeOnce = async id => {
    const { data } = await executeJmJob(id)
    if (data.code === 200) {
      message.success('开始执行')
    } else {
      message.error(data.msg)
    }
  }
  //删除
  const deleteTask = async id => {
    const { data } = await deleteJmJob(id)
    if (data.code === 200) {
      message.success('删除成功')
      fetchJmJobList()
    } else {
      message.error(data.msg)
    }
  }
  //Api调用
  const getApiUri = id => {
    const url = `${process.env.REACT_APP_BASE_URL}/jpt/jm_job_info/execute/${id}`
    copy(url)
    message.success('复制成功')
  }
  //编辑
  const editTask = id => {
    navigate('/task-mgmt/task-view', { state: { id } })
  }

  const watchLog = id => {
    navigate('/task-log', { state: { id } })
  }

  const cronNextDay = async val => {
    if (val.executionCycle !== '执行一次') {
      const { data } = await getCronNext(val.executionCycle)
      if (data.code === 200) {
        setDayData(data.data)
      } else {
        message.error(data.msg)
      }
    }
  }

  //获取列表
  const fetchJmJobList = async () => {
    setDataLoading(true)
    const { data } = await getJmJobList()
    if (data.code === 200) {
      const list = data.data.map(item => {
        return {
          key: item.id,
          taskName: item.name,
          taskType: item.type,
          taskTag: item.tag,
          executionCycle:
            item.cron_type === 1 ? '执行一次' : item.cron_expression,
          historicalRuntime: item.history,
          taskState: item.status === 1,
        }
      })
      setJogList(list)
    }
    setDataLoading(false)
  }
  useEffect(() => {
    fetchJmJobList()
  }, [])

  const columns = [
    {
      title: '任务名称',
      dataIndex: 'taskName',
      key: 'taskName',
    },
    {
      title: '任务类型',
      dataIndex: 'taskType',
      key: 'taskType',
    },
    {
      title: '任务标签',
      dataIndex: 'taskTag',
      key: 'taskTag',
    },
    {
      title: '执行周期',
      dataIndex: 'executionCycle',
      key: 'executionCycle',
    },
    {
      title: '下次执行时间',
      key: 'action',
      align: 'center',
      render: (_, record) =>
        record.executionCycle === '执行一次' ? (
          <span
            style={{
              color: '#bfbfbf',
              cursor: 'no-drop',
            }}>
            查看
          </span>
        ) : (
          <Popover content={popContent} trigger="click" placement="bottom">
            <span
              onClick={() => {
                cronNextDay(record)
              }}
              style={{
                color: '#1881DA',
                cursor: 'pointer',
              }}>
              查看
            </span>
          </Popover>
        ),
    },
    {
      title: '历史运行情况',
      dataIndex: 'historicalRuntime',
      key: 'historicalRuntime',
      align: 'center',
      render: list => (
        <div>
          {list.map((item, index) => {
            return (
              <img
                key={index}
                src={
                  item.executor_result === 0 || item.executor_result === 0
                    ? errImgUrl
                    : successImgUrl
                }
                alt=""
                className="icon_wrapper"
              />
            )
          })}
        </div>
      ),
    },
    {
      title: '状态',
      dataIndex: 'taskState',
      key: 'taskState',
      render: (val, record) => (
        <Switch
          checkedChildren="开启"
          unCheckedChildren="关闭"
          defaultChecked={val}
          onChange={checked => changeTaskState(checked, record.key)}
        />
      ),
    },
    {
      title: '操作',
      key: 'operation',
      render: (_, record) => (
        <Space size="middle">
          <span
            onClick={() => {
              runTimeOnce(record.key)
            }}
            style={{ color: '#1881DA', cursor: 'pointer' }}>
            执行一次
          </span>
          <span
            onClick={() => {
              watchLog(record.key)
            }}
            style={{ color: '#1881DA', cursor: 'pointer' }}>
            查看日志
          </span>
          <Dropdown
            overlay={moreMenu}
            trigger={['click']}
            onClick={() => setRecordKey(record.key)}>
            <EllipsisOutlined />
          </Dropdown>
        </Space>
      ),
    },
  ]

  const popContent = (
    <div>
      {dayData.map((item, index) => (
        <p key={index}>{item}</p>
      ))}
    </div>
  )

  const moreMenu = (
    <Menu
      items={[
        {
          label: (
            <span
              onClick={() => {
                editTask(recordKey)
              }}
              style={{ color: '#1881DA', cursor: 'pointer' }}>
              编辑
            </span>
          ),
          key: '0',
        },
        {
          label: (
            <Popconfirm
              title="确认删除？"
              okText="确认"
              cancelText="取消"
              onConfirm={() => deleteTask(recordKey)}>
              <span style={{ color: '#1881DA', cursor: 'pointer' }}>删除</span>
            </Popconfirm>
          ),
          key: '1',
        },
        {
          label: (
            <span
              onClick={() => {
                getApiUri(recordKey)
              }}
              style={{ color: '#1881DA', cursor: 'pointer' }}>
              API调用
            </span>
          ),
          key: '2',
        },
      ]}
    />
  )

  return (
    <TaskMgmtWrapper>
      <div className="link_btn">
        <Link to="/task-mgmt/task-view">
          <Button type="primary">创建离线任务</Button>
        </Link>
      </div>
      <Table
        columns={columns}
        dataSource={jobList}
        bordered
        loading={dataLoading}
      />
    </TaskMgmtWrapper>
  )
}

export default TaskMgmtView
