import { Alert, Button, Form, Select, Row, Tooltip } from 'antd'
import styled from 'styled-components'
import {
  getDagInfomation,
  getDataSourceInfo,
  getTableNameInfo,
  getTableSchemaInfo,
} from '../services'
import { useEffect, useState } from 'react'
import waringUrl from '../style/img/warning.png'
import syncUrl from '../style/img/sync.png'

const StepTwo = styled.div`
  display: flex;
  .StepTwo-content {
    margin: 20px 30px 0;
    padding: 20px;
    height: 450px;
    overflow-y: auto;
    border: 1px solid #c8d3e9;
    .dtsource {
      .ant-form-item-label {
        width: 120px;
      }
      .ant-select {
        width: 80%;
      }
    }
    .ant-row {
      margin-left: 0;
    }
    .content-item {
      .item-choseList {
        .ant-form-item-control-input {
          width: 160px;
          .ant-select {
            width: 100%;
          }
        }
      }
      .stateIcon {
        display: inline-flex;
        height: 32px;
        background-color: #fff;
        border: none;
      }
      .dataSourceType {
        margin-bottom: 24px;
        color: #7d7d7d;
      }
      .form {
        flex-wrap: nowrap;
      }
    }
  }
  .structure-detail {
    display: flex;
    width: 416px;
    height: 450px;
    border: 1px solid #c8d3e9;
    margin-top: 20px;
    margin-left: 60px;
    border-radius: 2px;
    flex-wrap: wrap;
    flex-direction: column;
    .detail-title {
      width: 100%;
      height: 44px;
      margin-top: 26px;
      color: #4a4a4a;
      border-bottom: 1px solid #c8d3e9;
      .datasourceFont {
        margin-left: 85px;
      }
      .schemaFont {
        margin-left: 115px;
      }
    }
    .detail-content {
      height: 350px;
      overflow: auto;
      ::-webkit-scrollbar {
        display: none;
      }
    }
    .detail-item {
      display: flex;
      .icon {
        display: inline-block;
        text-align: center;
        height: 40px;
        width: 50px;
        color: #c8d3e9;
        line-height: 50px;
      }
      .dataSource {
        margin-left: 41px;
        margin-top: 13px;
        width: 130px;
        height: 33px;
        line-height: 33px;
        text-align: center;
        background-color: #eef5fe;
        border-radius: 2px;
        text-overflow: ellipsis;
        overflow: hidden;
        white-space: nowrap;
        padding: 0 5px;
      }
      .Dagdatasource {
        margin-left: 10px;
      }
    }
  }
`
const FormItem = Form.Item
const { Option } = Select

export default function JobStepTwo({ InputDataForm, dagData, checkSync }) {
  //获取数据源节点
  const [nodes, setNodes] = useState([])
  //数据源名称
  const [datasources, setDatasources] = useState([])
  //表
  const [tableData, setTableData] = useState([])

  const [detailIsShow, setDetailIsShow] = useState('none')
  // 预览node
  const [selectedNode, setSelectedNode] = useState([])
  // 所有预览map
  const [nodeSchemas, setNodeSchemas] = useState({})

  const [nodeSchema, setNodeSchema] = useState({})

  //所有节点预览状态
  const [nodeStates, setNodeStates] = useState({})

  const [nodeState, setNodeState] = useState({})

  const getDagInfo = async dag_url => {
    const data = await getDagInfomation(dag_url)
    if (data.status === 200) {
      const dag_nodes = data.data.nodes.filter(item => item.op === 'datasource')
      setNodes(dag_nodes)
    }
  }

  const getDataSource = async () => {
    const params = {
      page: 1,
      size: 100,
    }
    const { data } = await getDataSourceInfo(params)
    if (data.code === 200) {
      setDatasources(data.data.items)
    }
  }
  const onDatasourcesSelect = async val => {
    if (val !== undefined) {
      const { data } = await getTableNameInfo(val)
      if (data.code === 200) {
        setTableData(data.data)
      }
    }
  }
  const onTableSelect = async (val, index) => {
    if (val !== undefined) {
      const { data } = await getTableSchemaInfo({
        id: InputDataForm.getFieldValue(`datasource${index}`),
        table_name: val,
      })
      if (data.code === 200) {
        const schemas = data.data.map(item => {
          const strs = item.split(':')
          return { dataField: strs[1], dataType: strs[2] }
        })
        handleSyncData(schemas, nodes[index]?.data.input_source, index)
      }
    }
  }

  const handleSyncData = (datasourceSchemas, nodeDataSchemas, index) => {
    const list = []
    const syncList = []
    nodeDataSchemas.forEach(item => {
      const syncItem = datasourceSchemas.find(
        val => val.dataField === item.dataField
      )
      if (syncItem) {
        const handleType = syncItem.dataType.split('(')[0]
        const result =
          item.dataType === handleType ||
          (item.dataType === 'string' && handleType === 'varchar')
        syncList.push({ datasource: syncItem, dag: item, result })
        datasourceSchemas = datasourceSchemas.filter(val => val !== syncItem)
      } else {
        list.push(item)
      }
    })
    if (list.length > datasourceSchemas.length) {
      list.forEach((item, index) => {
        syncList.push({
          datasource: item,
          dag: datasourceSchemas[index],
          result: false,
        })
      })
    } else {
      datasourceSchemas.forEach((item, index) => {
        syncList.push({
          datasource: item,
          dag: list[index],
          result: false,
        })
      })
    }
    if (list.length === 0) {
      setNodeState({ index, state: true })
    } else {
      setNodeState({ index, state: false })
    }

    setNodeSchema({ index, list: syncList })
  }

  const showStructure = index => {
    setDetailIsShow('block')
    setSelectedNode(nodeSchemas[index])
  }
  useEffect(() => {
    getDataSource()
  }, [])
  useEffect(() => {
    getDagInfo(dagData)
  }, [dagData])

  useEffect(() => {
    nodes.forEach((_, index) => {
      const datasource = InputDataForm.getFieldValue(`datasource${index}`)
      const table = InputDataForm.getFieldValue(`sourceTable${index}`)
      onDatasourcesSelect(datasource)
      onTableSelect(table, index)
    })
  }, [nodes])

  useEffect(() => {
    const states = { ...nodeStates }
    states[nodeState.index] = nodeState.state
    setNodeStates(states)
  }, [nodeState])

  useEffect(() => {
    const schemas = { ...nodeSchemas }
    schemas[nodeSchema.index] = nodeSchema.list
    setNodeSchemas(schemas)
  }, [nodeSchema])

  useEffect(() => {
    let result = true
    const keys = Object.keys(nodeStates).filter(item => item !== 'undefined')
    keys.forEach(key => {
      if (!nodeStates[key]) result = false
    })
    checkSync(result)
  }, [nodeStates])

  return (
    <StepTwo>
      <div className="StepTwo-content">
        <div className="content-item">
          <Form name="control-hooks" form={InputDataForm} className="form">
            {nodes.map((item, index) => (
              <div key={index}>
                <Row>
                  <FormItem
                    name={`datasource${index}`}
                    rules={[{ required: true, message: '请选择数据源!' }]}
                    label={item.name}
                    className="dtsource">
                    <Select allowClear onSelect={onDatasourcesSelect}>
                      {datasources.map(item => (
                        <Option key={item.id} value={item.id}>
                          {item.datasource_name}
                        </Option>
                      ))}
                    </Select>
                  </FormItem>
                  <FormItem
                    name={`sourceTable${index}`}
                    label="选择表"
                    rules={[{ required: true, message: '请选择表!' }]}
                    className="item-choseList">
                    <Select
                      allowClear
                      onSelect={val => onTableSelect(val, index)}>
                      {tableData.map(item => (
                        <Option key={item} value={item}>
                          {item}
                        </Option>
                      ))}
                    </Select>
                  </FormItem>
                  <Alert
                    type={nodeStates[index] ? 'success' : 'warning'}
                    showIcon
                    className="stateIcon"></Alert>
                  <Button type="primary" onClick={() => showStructure(index)}>
                    结构预览
                  </Button>
                </Row>
                <Row>
                  <div className="dataSourceType">
                    数据源类型:
                    {
                      datasources.find(
                        val =>
                          val.id ===
                          InputDataForm.getFieldValue(`datasource${index}`)
                      )?.datasource
                    }
                  </div>
                </Row>
              </div>
            ))}
          </Form>
        </div>
      </div>
      <div className="structure-detail" style={{ display: detailIsShow }}>
        <div className="detail-title">
          <span className="datasourceFont">数据源</span>
          <span className="schemaFont">Dag输入源结构</span>
        </div>
        <div className="detail-content">
          {selectedNode?.map((item, index) => (
            <div key={index} className="detail-item">
              <div
                className="dataSource"
                style={{
                  display: item?.datasource?.dataField ? 'block' : 'none',
                }}>
                <Tooltip
                  title={`${item?.datasource?.dataField}.${item?.datasource?.dataType}`}>
                  <span>{`${item?.datasource?.dataField}.${item?.datasource?.dataType}`}</span>
                </Tooltip>
              </div>
              <span className="icon">
                <img
                  src={item?.result ? syncUrl : waringUrl}
                  alt=""
                  style={{ width: '20px', height: '20px' }}
                />
              </span>
              <div
                className="dataSource Dagdatasource"
                style={{
                  display: item?.dag?.dataField ? 'block' : 'none',
                }}>
                <span>{`${item?.dag?.dataField}.${item?.dag?.dataType}`}</span>
              </div>
            </div>
          ))}
        </div>
      </div>
    </StepTwo>
  )
}
